PortalFu = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceEvent-2.0", "FuBarPlugin-2.0")

local L = AceLibrary("AceLocale-2.2"):new("PortalFu")
local tablet = AceLibrary("Tablet-2.0")
local dewdrop = AceLibrary("Dewdrop-2.0")

PortalFu.version = "1.0"
PortalFu.date = "2008-08-14"
PortalFu.hasIcon = true
PortalFu.canHideText = true
PortalFu.hasNoColor = true

PortalFu:RegisterDB("PortalFuDB", "PortalFuPerCharDB")
PortalFu:RegisterDefaults('profile', {
	showTag = true,
})


local function pairsByKeys(t)
	local a = {}
	for n in pairs(t) do
		table.insert(a, n)
	end
	table.sort(a, function(x, y)
		return x < y
	end)
	
	local i = 0
	local iter = function ()
		i = i + 1
		if a[i] == nil then
			return nil
		else
			return a[i], t[a[i]]
		end
	end
	return iter
end

local function findSpell(spellname)
	for i = 1,180 do
		local s = GetSpellName(i, BOOKTYPE_SPELL)
		if s == spellname then
			return i
		end
	end
end

---------------------------------------------
--             Addon Functions             --
---------------------------------------------

function PortalFu:OnInitialize()
	
	local transportSpells = {
		Alliance = {
			3561,  --TP:Stormwind
			3562,  --TP:Ironforge
			3565,  --TP:Darnassus
			32271, --TP:Exodar
			49359, --TP:Theramore
			33690, --TP:Shattrath
			10059, --P:Stormwind
			11416, --P:Ironforge
			11419, --P:Darnassus
			32266, --P:Exodar
			49360, --P:Theramore
			33691  --P:Shattrath
		},
		Horde = {
			3563,  --TP:Undercity
			3566,  --TP:Thunder Bluff
			3567,  --TP:Orgrimmar
			32272, --TP:Silvermoon
			49358, --TP:Stonard
			35715, --TP:Shattrath
			11418, --P:Undercity
			11420, --P:Thunder Bluff
			11417, --P:Orgrimmar
			32267, --P:Silvermoon
			49361, --P:Stonard
			35717  --P:Shattrath
		}
	}
	
	local _, class = UnitClass("player")
	if class == "MAGE" then
		self.ClassSpells = transportSpells[UnitFactionGroup("player")]
	end

	transportSpells = nil
end

function PortalFu:OnEnable()
	self.METHODS = {}
	self:UpdateSpells()
	self:RegisterEvent("SPELLS_CHANGED")
	self:SetDisplay()
end

function PortalFu:OnDisable()
	self.METHODS = nil
end

function PortalFu:SPELLS_CHANGED()
	self:UpdateSpells()
end

function PortalFu:OnMenuRequest(level,value)
	self:UpdateSpells()
	if level == 1 then
		for k,v in pairsByKeys(self.METHODS) do
			if v.secure then
				dewdrop:AddLine(
					'text', v.text,
					'secure', v.secure,
					'func', function() self:SetDisplay(k) end,
					'disabled', false,
					'closeWhenClicked', true
				)
			end
		end
		dewdrop:AddLine()
	end
end

function PortalFu:UpdateText()
	if method and self.METHODS[method] then
		local t = {}
		
		if self.db.profile.showTag then
			table.insert(t, self.METHODS[method].text)
		end
				
		self:SetText(table.concat(t, " "))
	else
		self:SetText(L["N/A"])
	end
end

function PortalFu:OnTooltipUpdate()
	local cat = tablet:AddCategory(
		'columns', 2,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0,
		'child_text2R', 1,
		'child_text2G', 1,
		'child_text2B', 1
	)
	cat:AddLine('text', L["Right-Click"], 'text2', L["to see list of spells"])
end

function PortalFu:UpdateSpells()
	if self.ClassSpells then
		for _,unTransSpell in ipairs(self.ClassSpells) do
			
			local spell = GetSpellInfo(unTransSpell)
			local spellid = findSpell(spell)
			
			if spellid then	
				self.METHODS[spell] = {
					spellid = spellid,
					text = spell,
					secure = {
						type = 'spell',
						spell = spell,
					}
				}
			end
		end
	end
end

function PortalFu:SetDisplay(method)
	local default = "Interface\\Icons\\INV_Misc_Rune_01"
		
	if self.METHODS[method] then
		
		local mtable = self.METHODS[method]

		self:SetIcon(GetSpellTexture(mtable.spellid, BOOKTYPE_SPELL))
	else
		self:SetIcon(default)
	end
	self:UpdateDisplay()
end
